(* For use by parser *)
let numScopes = ref 0
let nextScope unit = numScopes := !numScopes + 1 ; !numScopes

type arop = Plus | Minus | Times | Div | Divdiv | Exp | Percent (* Arithmetic operator *)
type logop = Eqeq | Neq | Gt | Lt | Gte | Lte | And | Or (* Logical operator *)
type unop = Neg | Not | Question (* Unary operator *)

type decl_type = Param | Comp | Var

type typ = IntT | FloatT | StringT | BoolT | ColorT | PairT
| FigureT of string option
| VoidT (* Used for function return type, empty list type *)
| FigDefT (* Used for figure definitions *)
| ListT of typ
| FuncT of typ * typ list (* return type * argument types *)

and expr =
    Int of int
  | Float of float
  | String of string
  | Bool of bool
  | Function of typ * (string * typ) list * block
  | List of expr list
  | Color of (string * string * string * float)
  | Pair of (expr * expr)
  | Subscript of expr * expr
  | Dot of expr * string
  | Id of string
  | Arop of expr * arop * expr
  | Logop of expr * logop * expr
  | Unop of unop * expr
  | Call of expr * expr list
  | FigInstance of string * (string * expr) list
and stmt = 
	  Expr of expr
  | Figure of int * string * decls (* int represents scope index *)
  | Decl of decls
  | Assign of expr * expr
  | Return of expr option
  | If of expr * block * block
  | While of expr * block
  | DoWhile of block * expr
and block = int * stmt list (* int represents scope index *)
and decls = (decl_type * string * typ option * expr option) list

type program = stmt list

module StringMap = Map.Make(String)

type figureInfo = {
  name: string; (* Figure name, e.g. Rect *)
  params: string list; (* All params *)
  rParams: string list; (* Required params *)
  comps: string list; (* All components *)
  vars: string list; (* All vars *)
}
type scope = {
  (* Map of symbols to types *)
  symbols: typ StringMap.t;
  (* Array index of outer scope.  None for global scope. *)
  outer: int option;
  (* Figure information, if scope is a figure *)
  figure: figureInfo option;
  (* For functions only. Map of symbols to int scope index *)
  capture: int StringMap.t option;
  (* For functions only. Return type. *)
  typ: typ option;
}
